@@ -122,6 +122,7 @@ show_options (ScreenshotDialog *dialog)
 {
   gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "options");
   gtk_stack_set_visible_child_name (GTK_STACK (dialog->header_stack), "options");
+  gtk_widget_show (GTK_WIDGET (dialog));
 }
 
 static void
@@ -417,9 +418,11 @@ screenshot_dialog_new (const char *app_id,
       g_autoptr(GAppInfo) info = NULL;
       id = g_strconcat (app_id, ".desktop", NULL);
       info = G_APP_INFO (g_desktop_app_info_new (id));
-      heading = g_strdup_printf (_("Share this screenshot with %s?"), g_app_info_get_display_name (info));
+      if (info)
+        heading = g_strdup_printf (_("Share this screenshot with %s?"), g_app_info_get_display_name (info));
     }
-  else
+
+  if (heading == NULL)
     heading = g_strdup (_("Share this screenshot with the requesting application?"));
 
   gtk_label_set_label (GTK_LABEL (dialog->heading), heading);
