@@ -50,6 +50,8 @@
 #include "access.h"
 #include "account.h"
 #include "email.h"
+#include "screencast.h"
+#include "remotedesktop.h"
 
 
 static GMainLoop *loop = NULL;
@@ -137,6 +139,18 @@ on_bus_acquired (GDBusConnection *connection,
       g_warning ("error: %s\n", error->message);
       g_clear_error (&error);
     }
+
+  if (!screen_cast_init (connection, &error))
+    {
+      g_warning ("error: %s\n", error->message);
+      g_clear_error (&error);
+    }
+
+  if (!remote_desktop_init (connection, &error))
+    {
+      g_warning ("error: %s\n", error->message);
+      g_clear_error (&error);
+    }
 }
 
 static void
